module Twine
  module Formatters
    class QKRStringSwift < Abstract
      def format_name
        'qkr_string_swift'
      end

      def extension
        '.swift'
      end

      def can_handle_directory?(path)
        true
      end

      def default_file_name
        'QKRString+extension.swift'
      end

      def determine_language_given_path(path)
        return @twine_file.language_codes[0]
      end

      def output_path_for_language(lang)
        ""
      end

      def read(io, lang)
      end

      def format_header(lang)
        "/**\n * Apple Strings File\n * Generated by Twine #{Twine::VERSION}\n */"
      end

      def format_sections(twine_file, lang)
        result = "import Foundation\n"
        result += "import QualiacKernel\n\n"
        result += "// swiftlint:disable identifier_name line_length superfluous_disable_command\n"
        result += "extension QKRString {\n"
        
        result += super + "\n"

        result += "}\n"
        result += "// swiftlint:enable identifier_name line_length superfluous_disable_command\n"
      end

      def format_section_header(section)
        "\t/********** #{section.name} **********/\n"
      end
      
      def format_key_value(definition, lang)
        key_value_pattern % { swift_key: format_swift_key(definition.key.dup), key: format_key(definition.key.dup) }
      end

      def key_value_pattern
        "\tvar %{swift_key}: String { return load(\"%{key}\", in: Bundle.main, table: \"Localizable\") }\n"
      end

      def format_swift_key(key)
        if ["class", "deinit", "enum", "func", "import", "init", "internal", "let", "operator", "private", "protocol",
            "public", "static", "struct", "subscript", "typealias", "var"].include? key
          key = "`#{key}`"
        end
        
        key
      end
      
      def format_key(key)
        escape_quotes(key)
      end

      def format_value(value)
        escape_quotes(value)
      end
    end
  end
end
